@@ -64,6 +64,8 @@ def Filter(expr, p):
 def Extend(p, expr, var):
     return CompValue('Extend', p=p, expr=expr, var=var)
 
+def Values(res):
+    return CompValue('values', res=res)
 
 def Project(p, PV):
     return CompValue('Project', p=p, PV=PV)
@@ -128,7 +130,7 @@ def triples(l):
     if (len(l) % 3) != 0:
         raise Exception('these aint triples')
     return reorderTriples((l[x], l[x + 1], l[x + 2])
-                          for x in range(0, len(l), 3))
+                          for x in list(range(0, len(l), 3)))
 
 
 def translatePName(p, prologue):
@@ -202,6 +204,9 @@ def translateExists(e):
         if isinstance(n, CompValue):
             if n.name in ('Builtin_EXISTS', 'Builtin_NOTEXISTS'):
                 n.graph = translateGroupGraphPattern(n.graph)
+                if n.graph.name == 'Filter':
+                    # filters inside (NOT) EXISTS can see vars bound outside
+                    n.graph.no_isolated_scope = True
 
     e = traverse(e, visitPost=_c)
 
@@ -421,26 +426,37 @@ def _findVars(x, res):
             res.add(x.var)
             return x  # stop recursion and finding vars in the expr
         elif x.name == 'SubSelect':
-            if x.projection: 
+            if x.projection:
                 res.update(v.var or v.evar for v in x.projection)
             return x
 
 
 def _addVars(x, children):
-    # import pdb; pdb.set_trace()
+    """
+    find which variables may be bound by this part of the query
+    """
     if isinstance(x, Variable):
         return set([x])
     elif isinstance(x, CompValue):
-        x["_vars"] = set(reduce(operator.or_, children, set()))
-        if x.name == "Bind":
-            return set([x.var])
-        elif x.name == 'SubSelect':
-            if x.projection:
-                s = set(v.var or v.evar for v in x.projection)
-            else: 
-                s = set()
+        if x.name == "RelationalExpression":
+            x["_vars"] = set()
+        elif x.name == "Extend":
+            # vars only used in the expr for a bind should not be included
+            x["_vars"] = reduce(operator.or_, [ child for child,part in zip(children,x) if part!='expr' ], set())
+
+        else:
+            x["_vars"] = set(reduce(operator.or_, children, set()))
+
+            if x.name == 'SubSelect':
+                if x.projection:
+                    s = set(v.var or v.evar for v in x.projection)
+                else:
+                    s = set()
+
+                return s
+
+        return x["_vars"]
 
-            return s
     return reduce(operator.or_, children, set())
 
 
@@ -468,7 +484,7 @@ def translateAggregates(q, M):
     #    select expr as ?var
     if q.projection:
         for v in q.projection:
-            if v.evar: 
+            if v.evar:
                 v.expr = traverse(v.expr, functools.partial(_sample, v=v.evar))
                 v.expr = traverse(v.expr, functools.partial(_aggs, A=A))
 
@@ -485,13 +501,13 @@ def translateAggregates(q, M):
 
     # sample all other select vars
     # TODO: only allowed for vars in group-by?
-    if q.projection: 
-        for v in q.projection: 
+    if q.projection:
+        for v in q.projection:
             if v.var:
                 rv = Variable('__agg_%d__' % (len(A) + 1))
                 A.append(CompValue('Aggregate_Sample', vars=v.var, res=rv))
                 E.append((rv, v.var))
-                
+
     return CompValue('AggregateJoin', A=A, p=M), E
 
 
@@ -512,7 +528,7 @@ def translateValues(v):
         for vals in v.value:
             res.append(dict(list(zip(v.var, vals))))
 
-    return CompValue('values', res=res)
+    return Values(res)
 
 
 def translate(q):
@@ -565,7 +581,7 @@ def translate(q):
     if q.valuesClause:
         M = Join(p1=M, p2=ToMultiSet(translateValues(q.valuesClause)))
 
-    if not q.projection: 
+    if not q.projection:
         # select *
         PV = list(VS)
     else:
@@ -579,7 +595,7 @@ def translate(q):
                     PV.append(v.evar)
 
                 E.append((v.expr, v.evar))
-            else: 
+            else:
                 raise Exception("I expected a var or evar here!")
 
     for e, v in E:
@@ -628,6 +644,12 @@ def simplify(n):
 
 def analyse(n, children):
 
+    """
+    Some things can be lazily joined.
+    This propegates whether they can up the tree
+    and sets lazy flags for all joins
+    """
+
     if isinstance(n, CompValue):
         if n.name == 'Join':
             n["lazy"] = all(children)
