@@ -326,9 +326,12 @@ def evalSlice(ctx, slice):
     # import pdb; pdb.set_trace()
     res = evalPart(ctx, slice.p)
     i = 0
-    while i < slice.start:
-        next(res)
-        i += 1
+    try:
+        while i < slice.start:
+            next(res)
+            i += 1
+    except StopIteration:
+        return
     i = 0
     for x in res:
         i += 1
@@ -436,7 +439,7 @@ def evalConstructQuery(ctx, query):
 
 def evalQuery(graph, query, initBindings, base=None):
 
-    initBindings = dict( ( Variable(k),v ) for k,v in initBindings.items() )
+    initBindings = dict( ( Variable(k),v ) for k,v in iter(initBindings.items()) )
 
     ctx = QueryContext(graph, initBindings=initBindings)
 
