@@ -139,7 +139,7 @@ def expandCollection(terms):
 # ------ TERMINALS --------------
 # [139] IRIREF ::= '<' ([^<>"{}|^`\]-[#x00-#x20])* '>'
 IRIREF = Combine(Suppress('<') + Regex(r'[^<>"{}|^`\\%s]*' % ''.join(
-    '\\x%02X' % i for i in range(33))) + Suppress('>'))
+    '\\x%02X' % i for i in list(range(33)))) + Suppress('>'))
 IRIREF.setParseAction(lambda x: rdflib.URIRef(x[0]))
 
 # [164] P_CHARS_BASE ::= [A-Z] | [a-z] | [#x00C0-#x00D6] | [#x00D8-#x00F6] | [#x00F8-#x02FF] | [#x0370-#x037D] | [#x037F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] | [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] | [#x10000-#xEFFFF]
