@@ -274,7 +274,7 @@ def evalUpdate(graph, update, initBindings={}):
 
     for u in update:
 
-        initBindings = dict( ( Variable(k),v ) for k,v in initBindings.items() )
+        initBindings = dict( ( Variable(k),v ) for k,v in iter(initBindings.items()) )
 
         ctx = QueryContext(graph, initBindings=initBindings)
         ctx.prologue = u.prologue
